@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-VERSION = "0.34"
+VERSION = "0.38"
 
 JAVA_EXE = "java"
 
@@ -87,11 +87,12 @@ owlready2.prop.AnnotationProperty      = AnnotationProperty
 owlready2.prop.Thing                   = Thing
 owlready2.prop.PropertyChain           = PropertyChain
 owlready2.prop._check_superclasses     = True
-
 owlready2.prop.ThingClass              = ThingClass
 owlready2.prop.And                     = And
 owlready2.prop.Or                      = Or
 owlready2.prop.OneOf                   = OneOf
+owlready2.prop.NamedIndividual         = NamedIndividual
+
 owlready2.annotation.Construct         = Construct
 
 owlready2.individual._keep_most_specific = _keep_most_specific
@@ -109,22 +110,22 @@ owlready2.class_construct.EntityClass = EntityClass
 
 del owlready2
 
-from owlready2.rule            import *
+from owlready2.rule import *
 
 LOADING.__exit__()
 
 # Not real property
-del owl_world._props["Property"]
-del owl_world._props["ObjectProperty"]
-del owl_world._props["DatatypeProperty"]
-del owl_world._props["FunctionalProperty"]
-del owl_world._props["InverseFunctionalProperty"]
-del owl_world._props["TransitiveProperty"]
-del owl_world._props["SymmetricProperty"]
-del owl_world._props["AsymmetricProperty"]
-del owl_world._props["ReflexiveProperty"]
-del owl_world._props["IrreflexiveProperty"]
-del owl_world._props["AnnotationProperty"]
+owl_world._props.pop("Property", None)
+owl_world._props.pop("ObjectProperty", None)
+owl_world._props.pop("DatatypeProperty", None)
+owl_world._props.pop("FunctionalProperty", None)
+owl_world._props.pop("InverseFunctionalProperty", None)
+owl_world._props.pop("TransitiveProperty", None)
+owl_world._props.pop("SymmetricProperty", None)
+owl_world._props.pop("AsymmetricProperty", None)
+owl_world._props.pop("ReflexiveProperty", None)
+owl_world._props.pop("IrreflexiveProperty", None)
+owl_world._props.pop("AnnotationProperty", None)
 
 default_world = IRIS = World()
 get_ontology  = default_world.get_ontology
