@@ -157,7 +157,7 @@ def _parse_date(s):
 
 def _parse_datetime(s):
   for format in ["%Y-%m-%dT%H:%M:%S", "%Y-%m-%dT%H:%M:%S%z", "%Y-%m-%dT%H:%M:%S.%f", "%Y-%m-%dT%H:%M:%S.%f%z"]:
-    try:    return datetime.datetime.strptime(s, format)
+    try:    return datetime.datetime.strptime(s.replace(" ", "T"), format)
     except: pass
   raise ValueError("Cannot parse ISO datetime '%s'!" % s)
 
@@ -178,6 +178,25 @@ _universal_abbrev_datatype(datetime.time,
                            lambda s: datetime.datetime.strptime(s, "%H:%M:%S").time(),
                            datetime.time.isoformat, "http://www.w3.org/2001/XMLSchema#time")
 
+def _parse_duration(s):
+  assert s.startswith("P")
+  days = seconds = 0
+  in_date = True
+  for number, letter in re.findall(r"([0-9.]+)([A-Z]+)", s):
+    number = float(number)
+    letter0 = letter[0]
+    if letter0 == "Y": days     +=  365.0 * number
+    if letter0 == "D": days     +=          number
+    if letter0 == "H": seconds  += 3600.0 * number
+    if letter0 == "S": seconds  +=          number
+    if letter0 == "M":
+      if in_date:      days     +=   30.0 * number
+      else:            seconds  +=   60.0 * number
+    if "T" in letter: in_date = False
+  return datetime.timedelta(days, seconds)
+
+def _format_duration(td):
+  return "P%sD%sS" % (td.days, td.seconds + td.microseconds / 1000000.0)
 
 def set_datatype_iri(datatype, iri):
   unparser = _universal_datatype_2_abbrev_unparser[datatype][1]
@@ -186,14 +205,16 @@ def set_datatype_iri(datatype, iri):
 
   
 def declare_datatype(datatype, iri, parser, unparser):
-  storid = _universal_abbrev(iri)
+  import sqlite3
+  if iri in _universal_iri_2_abbrev: storid = _universal_iri_2_abbrev[iri]
+  else:                              storid = _universal_abbrev(iri)
   from owlready2 import WORLDS
   for world in WORLDS:
-    if not world.graph.read_only:
-      world.graph.execute("INSERT OR IGNORE INTO resources VALUES (?,?)", (storid, iri))
-      #if world.graph._abbreviate_d:
-      #  world.graph._abbreviate_d  [iri]    = storid
-      #  world.graph._unabbreviate_d[storid] = iri
+    if (not world.graph.read_only):
+      try:
+        world.graph.execute("INSERT OR IGNORE INTO resources VALUES (?,?)", (storid, iri))
+      except sqlite3.ProgrammingError: # World has been closed
+        pass
       
   _universal_datatype_2_abbrev         [datatype] =  storid
   _universal_datatype_2_abbrev_unparser[datatype] = (storid, unparser)
@@ -285,9 +306,15 @@ owl_disjointunion            = _universal_abbrev("http://www.w3.org/2002/07/owl#
 #owlready_context_is_a        = _universal_abbrev("http://www.lesfleursdunormal.fr/static/_downloads/owlready_ontology.owl#context_is_a")
 owlready_concrete            = _universal_abbrev("http://www.lesfleursdunormal.fr/static/_downloads/owlready_ontology.owl#concrete")
 
-rdf_langstring = _universal_abbrev("http://www.w3.org/1999/02/22-rdf-syntax-ns#langString")
+rdf_langstring               = _universal_abbrev("http://www.w3.org/1999/02/22-rdf-syntax-ns#langString")
 _universal_abbrev_2_datatype[rdf_langstring] = locstr
 
+#xsd_duration                 = _universal_abbrev("http://www.w3.org/2001/XMLSchema#duration")
+_universal_abbrev_datatype(datetime.timedelta,
+                           _parse_duration,
+                           _format_duration,
+                           "http://www.w3.org/2001/XMLSchema#duration")
+
 issubclass_python = issubclass
 
 
