@@ -47,7 +47,7 @@ class PropertyClass(EntityClass):
   #     if DataProperty       in superclasses: nb_base += 1
   #     if AnnotationProperty in superclasses: nb_base += 1
   #     if nb_base > 1:
-  #       iri = "%s%s" % (obj_dict["namespace"].base_iri, name)
+  #       iri = "%s%s" % (obj_dict["namespace"]._base_iri, name)
   #       if (ObjectProperty in superclasses) and (DataProperty in superclasses):
   #         raise TypeError("Property '%s' is both an ObjectProperty and a DataProperty!" % iri)
   #       if (ObjectProperty in superclasses) and (AnnotationProperty in superclasses):
@@ -390,7 +390,36 @@ class PropertyClass(EntityClass):
     else:
       Prop._set_values_for_individual(entity, value)
       
+  def descendants(Prop, include_self = True, only_loaded = False, world = None):
+    if   Prop is ObjectProperty:
+      import owlready2
+      r = { prop for prop in (world or owlready2.default_world)._props.values() if issubclass(prop, ObjectProperty) and (prop.namespace._base_iri != "http://www.w3.org/2003/11/swrl#") and (not prop is bottomObjectProperty) }
+      if include_self: r.add(ObjectProperty)
+      return r
+    elif Prop is DataProperty:
+      import owlready2
+      r = { prop for prop in (world or owlready2.default_world)._props.values() if issubclass(prop, DataProperty) and (prop.namespace._base_iri != "http://www.w3.org/2003/11/swrl#") and (not prop is bottomDataProperty) }
+      if include_self: r.add(DataProperty)
+      return r
+    elif Prop is AnnotationProperty:
+      import owlready2
+      r = { prop for prop in (world or owlready2.default_world)._props.values() if issubclass(prop, AnnotationProperty) }
+      if include_self: r.add(AnnotationProperty)
+      return r
+    return EntityClass.descendants(Prop, include_self, only_loaded, world)
 
+  def subclasses(Prop, only_loaded = False, world = None):
+    if   Prop is ObjectProperty:
+      import owlready2
+      return { prop for prop in (world or owlready2.default_world)._props.values() if (ObjectProperty in prop.__bases__) and (prop.namespace._base_iri != "http://www.w3.org/2003/11/swrl#") and (not prop is bottomObjectProperty) }
+    elif Prop is DataProperty:
+      import owlready2
+      return { prop for prop in (world or owlready2.default_world)._props.values() if (DataProperty in prop.__bases__) and (prop.namespace._base_iri != "http://www.w3.org/2003/11/swrl#") and (not prop is bottomDataProperty) }
+    elif Prop is AnnotationProperty:
+      import owlready2
+      return { prop for prop in (world or owlready2.default_world)._props.values() if (AnnotationProperty in prop.__bases__) }
+      
+    return EntityClass.subclasses(Prop, only_loaded, world)
   
 _FUNCTIONAL_FOR_CACHE = weakref.WeakKeyDictionary()
 
@@ -873,7 +902,6 @@ def destroy_entity(e, undoable = False):
   if undoable: undoer_objs = []; undoer_datas = []; undoer_bnodes = []; undoer_relations = []
   else:        undoer_objs = undoer_datas = None; undoer_bnodes = None; undoer_relations = None
   
-  
   if   hasattr(e, "__destroy__"): e.__destroy__(undoer_objs, undoer_datas)
   
   elif isinstance(e, PropertyClass):
@@ -923,9 +951,8 @@ def destroy_entity(e, undoable = False):
     if o:
       for r in relations:
         if  (r == rdf_type) or (r == rdfs_subpropertyof) or (r == rdfs_subclassof):
-          #o.is_a.reinit([i for i in o.is_a if not i.storid in destroyed_storids])
           parents = [e.namespace.world._to_python(i) for i in e.namespace.world._get_obj_triples_sp_o(storid, r)]
-          o.is_a.reinit([i for i in parents if not i is None])
+          o.is_a.reinit([i for i in parents if not i is None and not i is NamedIndividual])
           if r == rdfs_subclassof:
             for Subclass in o.descendants(True, True): _FUNCTIONAL_FOR_CACHE.pop(Subclass, None)
             
@@ -964,7 +991,7 @@ def destroy_entity(e, undoable = False):
       for c,s,p,o in undoer_objs:
         c_2_onto[c]._add_obj_triple_spo(s,p,o)
       for c,s,p,o,d in undoer_datas:
-        c_2_onto[c]._add_data_triple_spo(s,p,o,d)
+        c_2_onto[c]._add_data_triple_spod(s,p,o,d)
       #e.namespace.world.graph.db.executemany("INSERT INTO objs VALUES (?,?,?,?)",    undoer_objs)
       #e.namespace.world.graph.db.executemany("INSERT INTO datas VALUES (?,?,?,?,?)", undoer_datas)
       e.namespace.world._entities[e.storid] = e
