@@ -20,7 +20,7 @@
 import sys, os, re, math
 from owlready2 import *
 from owlready2.sparql.parser import *
-from owlready2.sparql.func   import register_python_function, FuncSupport
+from owlready2.sparql.func   import register_python_builtin_functions, FuncSupport
 
 _RE_AUTOMATIC_INDEX = re.compile(r"([^ ]*?) USING AUTOMATIC.*\((.*?)\)")
 #_RE_NORMAL_INDEX    = re.compile(r"(.*?) AS (.*?) USING (COVERING )?INDEX (.*?) ")
@@ -45,7 +45,7 @@ class Translator(object):
     self.table_type_2_cols             = { "objs" : ["s", "p", "o"], "datas" : ["s", "p", "o", "d"], "quads" : ["s", "p", "o", "d"] , "one" : ["i"] }
     
     if not getattr(world.graph, "_has_sparql_func", False):
-      register_python_function(world)
+      register_python_builtin_functions(world)
       world.graph._has_sparql_func = True
       
   def make_translator(self):
@@ -68,6 +68,7 @@ class Translator(object):
       if max(prelim.recursive for prelim in self.preliminary_selects): sql += "RECURSIVE "
       sql += ",\n\n".join(prelim.sql() for prelim in self.preliminary_selects)
       sql += "\n\n"
+
     sql += self.main_query.sql()
     
     if self.solution_modifier[0]:
@@ -218,7 +219,7 @@ class Translator(object):
       #if isinstance(block, ExistsBlock): s.extra_sql = "IS NOT NULL"
       #else:                              s.extra_sql = "IS NULL"
       s.exists = isinstance(block, ExistsBlock)
-      s.vars   = nested_inside.vars
+      if nested_inside: s.vars = nested_inside.vars
       preliminary = False
       
     elif isinstance(block, NotExistsBlock):
@@ -260,7 +261,7 @@ class Translator(object):
       for bind in extra_binds: s.prepare_bind(bind)
       
       s.parse_selects(selects)
-      
+
       s.parse_triples(block)
       for bind in extra_binds: s.parse_bind(bind)
       
@@ -521,6 +522,8 @@ class PreparedModifyQuery(PreparedQuery):
     nb_match = 0
     if self.sql: resultss = PreparedQuery.execute(self, [params[i] for i in self.select_param_indexes])
     else:        resultss = [()]
+    
+    added_triples = []
     for results in set(resultss):
       nb_match += 1
       
@@ -544,7 +547,9 @@ class PreparedModifyQuery(PreparedQuery):
           elif type == "paramdatatype": triple.append(self.world._to_rdf(params[value])[1])
           else:                         triple.append(value)
         #print("ADD", insert, triple)
-        self.world._add_triple_with_update(self.ontology, *triple)
+        added_triples.append(triple)
+        
+    if added_triples: self.world._add_triples_with_update(self.ontology, added_triples)
     return nb_match
   
     
@@ -586,8 +591,8 @@ class Variable(object):
   
   def update_type(self, type):
     if   self.type == "quads": self.type = type
-    elif (type != self.type) and (type != "quads") :
-      raise ValueError("Variable %s cannot be both %s and %s!" % (self.name, self.type, type))
+    elif (type != self.type) and (type != "quads"):
+      raise ValueError("Variable %s cannot be both %s and %s!\n\n(NB if you are querying entities that are objects of a rdfs:label or comment relations, please add the following code to prevent Owlready from assuming that label and comment are data and not entities:\n    import owlready2.sparql.parser\n    owlready2.sparql.parser._DATA_PROPS = set()\n)" % (self.name, self.type, type))
     
 class Table(object):
   def __init__(self, query, name, type = "quads", index = None, join = ",", join_conditions = None):
@@ -627,13 +632,45 @@ class SQLQuery(FuncSupport):
     self.select_simple_union      = False
     self.optional                 = False
     
+    
   def __repr__(self): return "<%s '%s'>" % (self.__class__.__name__, self.sql())
-  
+
+  def _find_join_preceding_table(self, table):
+    if table.join == ",": return None
+    for condition in table.join_conditions:
+      parts = condition.split("=", 1)
+      if len(parts) == 2:
+        preceding = self.name_2_table.get(parts[1].split(".", 1)[0])
+        if preceding: return preceding
+        
   def sql(self):
     if self.tables:
       sql = """SELECT """
       if self.distinct: sql += "DISTINCT "
       sql += """%s FROM """ % (", ".join(str(column.binding) for column in self.columns))
+      
+      table_2_preceding = {}
+      for table in self.tables:
+        preceding = self._find_join_preceding_table(table)
+        if preceding: table_2_preceding[table] = preceding
+        
+      if table_2_preceding:
+        tables = list(self.tables)
+        for table, preceding in table_2_preceding.items():
+          table_i     = tables.index(table)
+          preceding_i = tables.index(preceding)
+          if table_i < preceding_i:
+            tables.remove(table)
+            tables.insert(tables.index(preceding) + 1, table)
+        self.tables = tables
+        
+      if self.tables[0].join != ",": # Cannot JOIN on the first table
+        for table in self.tables:
+          if table.join == ",":
+            self.tables.remove(table)
+            self.tables.insert(0, table)
+            break
+          
       for table in self.tables:
         if not table is self.tables[0]: sql += " %s " % table.join
         sql += table.sql()
@@ -720,9 +757,8 @@ class SQLQuery(FuncSupport):
       self.conditions.append(sub)
       if _DEPRIORIZE_SUBQUERIES_OPT and not ("one" in self.name_2_table): Table(self, "one", "one")
     else:
-      if (not sub.optional) and (len(sub.columns) == 1): # Specific optimization with IN operator
-        self.create_in_conditions(self.conditions, sub.columns[0].var, sub)
-      else:
+      ok = self.try_create_in_conditions(self.conditions, sub.columns[0].var, sub) # Try specific optimization with IN operator
+      if not ok:
         table = Table(self, "p%s" % self.translator.next_table_id, sub.name)
         table.subquery = sub
         self.translator.next_table_id += 1
@@ -737,7 +773,7 @@ class SQLQuery(FuncSupport):
           var.update_type(column.type)
           if not column.name.endswith("d"):
             self.create_conditions(conditions, table, column.name, var)
-              
+
   def parse_triples(self, triples):
     if self.triples: raise ValueError("Cannot parse triples twice!")
     self.block = triples
@@ -756,9 +792,10 @@ class SQLQuery(FuncSupport):
     
     for triple in list(self.triples): # Pass 1: Determine var type and prop type
       if isinstance(triple, (Bind, Filter, Block)): continue
-      s, p, o = triple
-      
       triple.local_table_type = triple.table_type
+      
+      if triple.optional: continue # Optional => cannot be used to restrict variable type
+      s, p, o = triple
       if s.name == "VAR":
         var = self.parse_var(s)
         var.update_type("objs")
@@ -853,8 +890,65 @@ class SQLQuery(FuncSupport):
         if o.name == "VAR":
           o = self.parse_var(o)
           o.bindings.insert(0, """IN (SELECT o FROM %s%s)""" % (triple.local_table_type, extra))
-          
+
+    has_optional_triple = False
+    for triple in self.triples:
+      if getattr(triple, "optional", False):
+        has_optional_triple = True
+        break
+        
     conditions = self.conditions
+    if isinstance(triples, TripleBlockWithStatic):
+      for static in triples.static_valuess:
+        if isinstance(static, StaticBlock):
+          if self.raw_selects == "*":
+            static.vars = sorted(static.all_vars)
+          else:
+            used_vars = set(self.vars)
+            used_vars.update(var.name for var in self.vars_needed_for_select)
+            static.vars = sorted(var for var in static.all_vars if (var in used_vars))
+            
+          for var in static.vars:
+            var = static.translator.main_query.vars.get(var)
+            if var: static.types.append(var.type)
+            else:   static.types.append("quads")
+              
+          var_2_columns = defaultdict(list)
+          for column in static.translator.main_query.columns:
+            var_2_columns[column.var].append(column)
+            
+          static.translator.main_query.raw_selects = static.vars
+          static.translator.main_query.columns = []
+          static.translator.main_query.finalize_columns()
+          static.translator.solution_modifier = [None, None, None, None, None]
+          q = static.translator.finalize()
+          static.valuess = list(q._execute_sql())
+          
+        if (not has_optional_triple) and len(static.vars) == 1: # This optimization is not supported with optional blocks
+          var = self.parse_var(static.vars[0])
+          var.update_type(static.types[0])
+          sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
+          if sql is None:
+            var.bindings.append("IN (%s)" % ",".join(str(value[0]) for value in static.valuess))
+            var.static = static.valuess
+            var.type = "objs"
+          else:
+            conditions.append("%s IN (%s)" % (sql, ",".join(str(value[0]) for value in static.valuess)))
+            
+        else:
+          prelim = SQLStaticValuesPreliminaryQuery(self.translator, "static%s" % (len(self.translator.preliminary_selects) + 1), static)
+          self.translator.preliminary_selects.append(prelim)
+          table = Table(self, prelim.name, prelim.name)
+          for i, (var, type) in enumerate(zip(static.vars, static.types)):
+            var = self.parse_var(var)
+            var.update_type(type)
+            sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
+            if sql is None:
+              var.bindings.append("%s.col%s_o" % (prelim.name, i + 1))
+            else:
+              var.bindings.insert(0, "%s.col%s_o" % (prelim.name, i + 1)) # Favor static binding, because it is never optional
+              conditions.append("%s=%s.col%s_o" % (sql, prelim.name, i + 1))
+    
     for triple in self.triples: # Pass 6: create triples tables and conditions
       if   isinstance(triple, Bind):
         self.parse_bind(triple)
@@ -886,57 +980,63 @@ class SQLQuery(FuncSupport):
       else:
         conditions = self.conditions
       self.translator.next_table_id += 1
-
+      
       if triple.consider_s: self.create_conditions(conditions, table, "s", s)
       if triple.consider_p: self.create_conditions(conditions, table, "p", p, triple.likelihood_p)
       if triple.consider_o: self.create_conditions(conditions, table, "o", o, triple.likelihood_o)
       
       if p.modifier == "+": conditions.append("%s.nb>0"  % table.name)
       
-      
-    if isinstance(triples, SimpleTripleBlock):
-      for static in triples.static_valuess:
-        if isinstance(static, StaticBlock):
-          if self.raw_selects == "*":
-            static.vars = sorted(static.all_vars)
-          else:
-            used_vars = set(self.vars)
-            used_vars.update(var.name for var in self.vars_needed_for_select)
-            static.vars = sorted(var for var in static.all_vars if (var in used_vars))
-          var_2_columns = defaultdict(list)
-          for column in static.translator.main_query.columns:
-              var_2_columns[column.var].append(column)
-              
-          static.translator.main_query.raw_selects = static.vars
-          static.translator.main_query.columns = []
-          static.translator.main_query.finalize_columns()
-          static.translator.solution_modifier = [None, None, None, None, None]
-          q = static.translator.finalize()
-          static.valuess = list(q._execute_sql())
-          
-        if len(static.vars) == 1:
-          var = self.parse_var(static.vars[0])
-          sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
-          if sql is None:
-            var.bindings.append("IN (%s)" % ",".join(str(value[0]) for value in static.valuess))
-            var.static = static.valuess
-            var.type = "objs"
-          else:
-            conditions.append("%s IN (%s)" % (sql, ",".join(str(value[0]) for value in static.valuess)))
-            #conditions.append("LIKELIHOOD(%s IN (%s), %s)" % (sql, ",".join(str(value) for value in static.valuess),
-            #                                                  0.35 + 0.1 * (1.0 - math.exp(-len(static.valuess) / 100.0)))) # Favor statics with few elements
+    # if isinstance(triples, TripleBlockWithStatic):
+    #   for static in triples.static_valuess:
+    #     if isinstance(static, StaticBlock):
+    #       if self.raw_selects == "*":
+    #         static.vars = sorted(static.all_vars)
+    #       else:
+    #         used_vars = set(self.vars)
+    #         used_vars.update(var.name for var in self.vars_needed_for_select)
+    #         static.vars = sorted(var for var in static.all_vars if (var in used_vars))
             
-        else:
-          prelim = SQLStaticValuesPreliminaryQuery(self.translator, "static%s" % (len(self.translator.preliminary_selects) + 1), static)
-          self.translator.preliminary_selects.append(prelim)
-          table = Table(self, prelim.name, prelim.name)
-          for i, var in enumerate(static.vars):
-            var = self.parse_var(var)
-            sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
-            if sql is None:
-              var.bindings.append("%s.col%s_o" % (prelim.name, i + 1))
-            else:
-              conditions.append("%s=%s.col%s_o" % (sql, prelim.name, i + 1))
+    #       for var in static.vars:
+    #         var = static.translator.main_query.vars.get(var)
+    #         if var: static.types.append(var.type)
+    #         else:   static.types.append("quads")
+              
+    #       var_2_columns = defaultdict(list)
+    #       for column in static.translator.main_query.columns:
+    #         var_2_columns[column.var].append(column)
+            
+    #       static.translator.main_query.raw_selects = static.vars
+    #       static.translator.main_query.columns = []
+    #       static.translator.main_query.finalize_columns()
+    #       static.translator.solution_modifier = [None, None, None, None, None]
+    #       q = static.translator.finalize()
+    #       static.valuess = list(q._execute_sql())
+          
+    #     if (not [table for table in self.tables if table.join != ","]) and len(static.vars) == 1: # This optimization is not supported with optional blocks
+    #       var = self.parse_var(static.vars[0])
+    #       var.update_type(static.types[0])
+    #       sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
+    #       if sql is None:
+    #         var.bindings.append("IN (%s)" % ",".join(str(value[0]) for value in static.valuess))
+    #         var.static = static.valuess
+    #         var.type = "objs"
+    #       else:
+    #         conditions.append("%s IN (%s)" % (sql, ",".join(str(value[0]) for value in static.valuess)))
+            
+    #     else:
+    #       prelim = SQLStaticValuesPreliminaryQuery(self.translator, "static%s" % (len(self.translator.preliminary_selects) + 1), static)
+    #       self.translator.preliminary_selects.append(prelim)
+    #       table = Table(self, prelim.name, prelim.name)
+    #       for i, (var, type) in enumerate(zip(static.vars, static.types)):
+    #         var = self.parse_var(var)
+    #         var.update_type(type)
+    #         sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
+    #         if sql is None:
+    #           var.bindings.append("%s.col%s_o" % (prelim.name, i + 1))
+    #         else:
+    #           var.bindings.insert(0, "%s.col%s_o" % (prelim.name, i + 1)) # Favor static binding, because it is never optional
+    #           conditions.append("%s=%s.col%s_o" % (sql, prelim.name, i + 1))
               
               
   def get_fix_levels(self, vars0, exclude_triple = None):
@@ -1023,17 +1123,33 @@ class SQLQuery(FuncSupport):
           condition = "%s.%s%s%s"  % (table.name, n, operator, sql)
           if likelihood is None: conditions.append(condition)
           else:                  conditions.append("LIKELIHOOD(%s, %s)" % (condition, likelihood))
-      if not sql_d is None:
+          
+      if (not sql_d is None) and (sql_d != 0) and (n != "s") and (n != "p") and (table.name != "objs"):
         conditions.append("%s.%sd=%s" % (table.name, n[:-1], sql_d)) # Datatype part
         
       if isinstance(x, rply.Token) and (x.name == "VAR"): x = self.vars[x.value]
       if isinstance(x, Variable):
         if not x.initial_query: x.initial_query = self
         x.bindings.append("%s.%s" % (table.name, n))
-        
-  def create_in_conditions(self, conditions, x, prelim):
+    
+  def try_create_in_conditions(self, conditions, x, prelim):
+    if prelim.optional or (len(prelim.columns) != 1) or (x is None): return False
+
     assert len(prelim.columns) == 1
-    if isinstance(x, rply.Token): assert x.name == "VAR"
+    if isinstance(x, rply.Token):
+      assert x.name == "VAR"
+      x_varname = x.value
+    else:
+      x_varname = x
+      
+    for triple in self.triples: # Verify if the variable is used outside the prelim query -- otherwise, the IN optimization cannot be used
+      if isinstance(triple, Triple):
+        if isinstance(triple[0], rply.Token) and (triple[0].name == "VAR") and (triple[0].value == x_varname): break
+        if isinstance(triple[1], rply.Token) and (triple[1].name == "VAR") and (triple[1].value == x_varname): break
+        if isinstance(triple[2], rply.Token) and (triple[2].name == "VAR") and (triple[2].value == x_varname): break
+    else:
+      return False
+    
     column = prelim.columns[0]
     var = self.parse_var(x)
     sql, sql_type, sql_d, sql_d_type = self._to_sql(var)
@@ -1043,7 +1159,7 @@ class SQLQuery(FuncSupport):
       var.in_select = prelim
     else:
       conditions.append("%s IN (SELECT %s FROM %s)" % (sql, column.name, prelim.name))
-      
+    return True  
       
   def is_fixed(self, x):
     if x.name != "VAR": return True
@@ -1055,7 +1171,7 @@ class SQLQuery(FuncSupport):
   def finalize_columns(self):
     selected_parameter_index = 0
     i = j = 0
-
+    
     if   self.raw_selects == "*": selects = [self.vars[var] for var in self.block.get_ordered_vars() if not var.startswith("??")]      
     elif self.raw_selects:        selects = self.raw_selects
     elif self.tables:
@@ -1099,12 +1215,19 @@ class SQLQuery(FuncSupport):
         else:
           raise ValueError("Cannot select '%s'!" % select)
         
+      if isinstance(sql, str) and sql.startswith("IN "):
+        if   not "," in sql: sql = sql[4 : -1]
+        elif len(selects) == 1: pass # Ok
+        else:
+          raise ValueError("Cannot SELECT a variable that appears only in a VALUES clause with multiple values (not yet supported).")
+        
       self.columns.append(Column(var_name, sql_type, sql, "col%s_o" % i, j)); j += 1
       if not sql_d is None:
         self.columns.append(Column(var_name, sql_d_type, sql_d, "col%s_d" % i, j)); j += 1
-        
+
     if self.preliminary:
       self.translator.table_type_2_cols[self.name] = [column.name for column in self.columns]
+
     
   def set_column_names(self, names):
     for column, name in zip(self.columns, names): column.name = name
@@ -1119,7 +1242,6 @@ class SQLQuery(FuncSupport):
     elif isinstance(x, Variable):
       if not x.bindings: return None, None, None, None
       binding = x.get_binding(self)
-      
       if   x.type == "objs":  return binding, "objs", None, None
       else:
         if not x.fixed_datatype is None:
@@ -1182,7 +1304,9 @@ class SQLCompoundQuery(object):
       for i, column in enumerate(query.columns): # Re-index columns
         column.index = i
     self.columns = self.queries[0].columns
-
+    
+    self.translator.table_type_2_cols[self.name] = [column.name for column in self.columns]
+    
     
 class SQLNestedQuery(SQLQuery):
   def __init__(self, name):
@@ -1192,17 +1316,12 @@ class SQLNestedQuery(SQLQuery):
   def finalize_columns(self):
     self.columns = [Column(None, "datas", "1", "col1_o", 1)]
     
-  def sql(self):
-    #extra_sql = self.extra_sql
-    #self.extra_sql = ""
-    
-    #sql = "(%s)" % SQLQuery.sql(self)
-    
-    #if extra_sql: sql += " %s" % extra_sql
-    #self.extra_sql = extra_sql
-    
+  #def sql(self):
+  #  return "SELECT WHERE %s" % self.nested_sql()
+  
+  def nested_sql(self):
     sql = SQLQuery.sql(self)
-
+    
     if _DEPRIORIZE_SUBQUERIES_OPT:
       if   self.exists == True:  sql =     "one.i= EXISTS(%s)" % sql
       elif self.exists == False: sql = "NOT one.i= EXISTS(%s)" % sql
@@ -1211,10 +1330,10 @@ class SQLNestedQuery(SQLQuery):
       if   self.exists == True:  sql =     "EXISTS(%s)" % sql
       elif self.exists == False: sql = "NOT EXISTS(%s)" % sql
       else:                      sql = "(%s)" % sql
-      
+
     return sql
   
-  def __str__(self): return self.sql()
+  def __str__(self): return self.nested_sql()
   
 
 class SQLRecursivePreliminaryQuery(SQLQuery):
@@ -1300,14 +1419,19 @@ class SQLStaticValuesPreliminaryQuery(object):
     self.translator    = translator
     self.name          = name
     self.static_values = static_values
-    self.translator.table_type_2_cols[self.name] = ["col%s_o" % (i+1) for i in range(len(self.static_values.valuess[0]))]
+    
+    if self.static_values.valuess:
+      self.nb_value = len(self.static_values.valuess[0])
+    else:
+      raise ValueError("VALUES clause cannot be empty (this also applies to STATIC block returning no results)!")
+    self.translator.table_type_2_cols[self.name] = ["col%s_o" % (i+1) for i in range(self.nb_value)]
     
     self.columns = []
-    for i in range(len(self.static_values.valuess[0])):
+    for i in range(self.nb_value):
       self.columns.append(Column(static_values.vars[i], "objs", "xxx", "col%s_o" % (i+1), i))
-    
+      
   def sql(self):
     return "%s(%s) AS (VALUES %s)" % (self.name,
-                                      ",".join("col%s_o" % (i+1) for i in range(len(self.static_values.valuess[0]))),
+                                      ",".join("col%s_o" % (i+1) for i in range(self.nb_value)),
                                       ",".join("(%s)" % ",".join(str(value) for value in values) for values in self.static_values.valuess) )
 
